/*
 * Copyright 2017 aquenos GmbH.
 * All rights reserved.
 * 
 * Please refer to the file NOTICE.txt in the root of the source distribution or
 * in the META-INF directory of the binary archive for licensing information.
 */

package com.aquenos.epics.jackie.diirt.datasource.internal;

import org.diirt.datasource.ValueCache;
import org.diirt.vtype.VString;

import com.aquenos.epics.jackie.common.value.ChannelAccessTimeChar;
import com.aquenos.epics.jackie.common.value.ChannelAccessValueType;
import com.aquenos.epics.jackie.diirt.vtype.JackieVStringFromByteArray;

/**
 * Type adapter that convert from {@link ChannelAccessTimeChar} to
 * {@link VString}. This adapter is not intended as the standard adapter for
 * {@link ChannelAccessTimeChar}s. It should only be used if such an array is
 * supposed to be treated as a long string.
 * 
 * @author Sebastian Marsching
 */
public class ArrayCharAsStringTypeAdapter
        extends JackieDataSourceTypeAdapter<VString> {

    /**
     * Single instance of this adapter.
     */
    public static final ArrayCharAsStringTypeAdapter INSTANCE = new ArrayCharAsStringTypeAdapter();

    private ArrayCharAsStringTypeAdapter() {
        super(false, ChannelAccessValueType.DBR_CHAR, VString.class);
    }

    @Override
    protected VString convertValue(JackieConnectionPayload connection,
            JackieMessagePayload message) {
        // We can safely make a cast here because this adapter is only used if
        // the channel's native type is DBR_CHAR and we thus know that the
        // monitor returns a DBR_TIME_CHAR.
        ChannelAccessTimeChar timeValue = (ChannelAccessTimeChar) message
                .getTimeValue();
        return new JackieVStringFromByteArray(timeValue,
                connection.getDisconnectTime(), connection.getCharset());
    }

    @Override
    public int match(ValueCache<?> cache, JackieConnectionPayload connection) {
        // The checks made by the super-class do not work for us because it
        // tries to match arrays with arrays and scalars with scalars, but we
        // want to match an array with a scalar.
        if (cache.getType().isAssignableFrom(VString.class)
                && ChannelAccessValueType.DBR_CHAR == connection
                        .getNativeDataType()
                && connection.isLongString()) {
            // We want this adapter to be preferred over the generic one (which
            // would treat the waveform as an array of bytes, so we use a
            // priority of one.
            return 1;
        } else {
            return 0;
        }
    }

}
