/*
 * Copyright 2017 aquenos GmbH.
 * All rights reserved.
 * 
 * Please refer to the file NOTICE.txt in the root of the source distribution or
 * in the META-INF directory of the binary archive for licensing information.
 */

package com.aquenos.epics.jackie.diirt.datasource.internal;

import org.diirt.vtype.VIntArray;

import com.aquenos.epics.jackie.common.value.ChannelAccessControlsLong;
import com.aquenos.epics.jackie.common.value.ChannelAccessTimeLong;
import com.aquenos.epics.jackie.common.value.ChannelAccessValueType;
import com.aquenos.epics.jackie.diirt.vtype.JackieVIntArray;

/**
 * Type adapter that convert from {@link ChannelAccessControlsLong} and
 * {@link ChannelAccessTimeLong} to {@link VIntArray}.
 * 
 * @author Sebastian Marsching
 */
public class ArrayLongTypeAdapter
        extends JackieDataSourceTypeAdapter<VIntArray> {

    /**
     * Single instance of this adapter.
     */
    public static final ArrayLongTypeAdapter INSTANCE = new ArrayLongTypeAdapter();

    private ArrayLongTypeAdapter() {
        super(true, ChannelAccessValueType.DBR_LONG, VIntArray.class);
    }

    @Override
    protected VIntArray convertValue(JackieConnectionPayload connection,
            JackieMessagePayload message) {
        // We can safely make a cast here because this adapter is only used if
        // the channel's native type is DBR_LONG and we thus know that the
        // monitors return a DBR_CTRL_LONG and DBR_TIME_LONG.
        ChannelAccessControlsLong controlsValue = (ChannelAccessControlsLong) message
                .getControlsValue();
        ChannelAccessTimeLong timeValue = (ChannelAccessTimeLong) message
                .getTimeValue();
        return new JackieVIntArray(controlsValue, timeValue,
                connection.getDisconnectTime());
    }

}
